/***************************************************************************/
/* Name        :FTag.cmd V1.0                                              */
/* Sprache     :Rexx (OS/2)                                                */
/* Autor       :Thorsten Ahlers                                            */
/*              Neues Land 34                                              */
/*              21522 Hohnstorf                                            */
/* Datum       :01.11.1994                                                 */
/* Funktion    :Berechnung der variablen Feiertage eines beliebigen Jahres */
/* Aufruf      :FTag <Jahreszahl>                                          */
/***************************************************************************/

/****************************************************************************
 Hauptprogramm
****************************************************************************/

parse arg jahr

if jahr="" then              /* wenn keine Parameter aktuelles Jahr setzen */
  jahr=date(S)/10000%1

if jahr < date(S)/10000%1 then
  WORD='war'
else
  WORD='ist'

call TAG_MONAT (OSTERSONNTAG(jahr) - 2), jahr
say 'Karfreitag 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT OSTERSONNTAG(jahr), jahr
say 'Ostersonntag 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (OSTERSONNTAG(jahr) + 1), jahr
say 'Ostermontag 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (OSTERSONNTAG(jahr) + 39), jahr
say 'Himmelfahrt 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (OSTERSONNTAG(jahr) + 49), jahr
say 'Pfingstsonntag 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (OSTERSONNTAG(jahr) + 50), jahr
say 'Pfingstmontag 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (337 + SCHALTJAHR(jahr) - WOCHENTAG(24, 12, jahr)), jahr
say 'Der 1. Advent 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (344 + SCHALTJAHR(jahr) - WOCHENTAG(24, 12, jahr)), jahr
say 'Der 2. Advent 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (351 + SCHALTJAHR(jahr) - WOCHENTAG(24, 12, jahr)), jahr
say 'Der 3. Advent 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (358 + SCHALTJAHR(jahr) - WOCHENTAG(24, 12, jahr)), jahr
say 'Der 4. Advent 'WORD' der 'TAG'.'MONAT'.'jahr

call TAG_MONAT (302 + SCHALTJAHR(jahr) + TAG), jahr
say 'Bus und Bettag 'WORD' der 'TAG'.'MONAT'.'jahr

return(0)


/****************************************************************************
 Diese Funktion ermittelt ob Schaltjahr oder nicht
 JA = 1   Nein = 0
****************************************************************************/

SCHALTJAHR: procedure
  parse arg jahr

  return(0 = jahr//4 & 0 < jahr//100 | 0 = jahr//400)

/****************************************************************************
 Diese Funktion berechnet den Wochentag in Abhnigkeit vom Datum
 Der Rckgabewert liegt zwischen 0 fr Sonntag und 6 fr Samstag
 Die alternative zu der Rexxfunktion DATE(B)//7
****************************************************************************/

WOCHENTAG: procedure
  parse arg tag, monat, jahr

  e = 0.6 + (1 / monat)
  e = e % 1              /* Nur ganzzahligen Anteil */
  m = monat + (12 * e)
  j = jahr - e
  a = 13 * (m + 1) % 5
  b = 5 * j % 4
  c = j % 100
  d = j % 400

  return((a + b + d + tag - c - 1) // 7)

/****************************************************************************
 Diese Funktion berechnet den Ostersonntag nach dem Mathematiker C.F.Gau
 Der Rckgabewert ist der Tag im Jahr
****************************************************************************/

OSTERSONNTAG: procedure
  parse arg jahr

  a = jahr // 19
  b = jahr // 4
  c = jahr // 7
  k = jahr % 100
  p = (13 + (8 * k)) % 25
  q = k % 4
  m = (15 - p + k - q) // 30
  n = (4 + k - q) // 7
  d = ((19 * a) + m) // 30
  E = ((2 * b) + (4 * c) + (6 * d) + n) // 7

  if(e = 6 & k = 28 & a > 10) then
    jtag = 108 + SCHALTJAHR(jahr)
  else if(e = 6 & d = 29) then
    jtag = 109 + SCHALTJAHR(jahr)
  else
    jtag = 81 + d + e + SCHALTJAHR(jahr)

  return(jtag)

/****************************************************************************
 Diese Procedure berechnet aus den Jahrestag Tag und Monat
****************************************************************************/

TAG_MONAT: procedure expose Tag Monat
  parse arg jtag, jahr

  if(jtag - 31 < 0 | jtag - 31 = 0) then do
    TAG = jtag
    MONAT = 1
    return(0)
    end  /* Do */

  jtag = jtag - 31

  if(jtag - 28 - SCHALTJAHR(jahr) < 0 | jtag - 28 - SCHALTJAHR(jahr) = 0) then do
    TAG = jtag
    MONAT = 2
    return(0)
    end  /* Do */

  jtag = jtag - 28 - SCHALTJAHR(jahr)

  if(jtag - 31 < 0 | jtag - 31 = 0) then do
    TAG = jtag
    MONAT = 3
    return(0)
    end  /* Do */

  jtag = jtag - 31

  if(jtag - 30 < 0 | jtag - 30 = 0) then do
    TAG = jtag
    MONAT = 4
    return(0)
    end  /* Do */

  jtag = jtag - 30

  if(jtag - 31 < 0 | jtag - 31 = 0) then do
    TAG = jtag
    MONAT = 5
    return(0)
    end  /* Do */

  jtag = jtag - 31

  if(jtag - 30 < 0 | jtag - 30 = 0) then do
    TAG = jtag
    MONAT = 6
    return(0)
    end  /* Do */

  jtag = jtag - 30

  if(jtag - 31 < 0 | jtag - 31 = 0) then do
    TAG = jtag
    MONAT = 7
    return(0)
    end  /* Do */

  jtag = jtag - 31

  if(jtag - 31 < 0 | jtag - 31 = 0) then do
    TAG = jtag
    MONAT = 8
    return(0)
    end  /* Do */

  jtag = jtag - 31

  if(jtag - 30 < 0 | jtag - 30 = 0) then do
    TAG = jtag
    MONAT = 9
    return(0)
    end  /* Do */

  jtag = jtag - 30

  if(jtag - 31 < 0 | jtag - 31 = 0) then do
    TAG = jtag
    MONAT = 10
    return(0)
    end  /* Do */

  jtag = jtag - 31

  if(jtag - 30 < 0 | jtag - 30 = 0) then do
    TAG = jtag
    MONAT = 11
    return(0)
    end  /* Do */

  jtag = jtag - 30

    TAG = jtag
    MONAT = 12

  return(0)

