/*********************************************************************/
/* UX2OS.CMD - (c) 1994 D. Zimmer-Bentin                             */
/*                                                                   */
/* Aufruf:                                                           */
/*   UX2OS InputFile OutputFile [/Replace | /Noreplace]              */
/*                                                                   */
/* Zweck:                                                            */
/*   In manchen Readme-Dokumenten wird statt CRLF am Zeilenende nur  */
/*   LF verwendet. UX2OS ersetzt jedes LF, das nicht auf ein CR      */
/*   folgt, durch ein CRLF. Dateinamen mit "Luecken" (bei HPFS)      */
/*   werden nicht unterstuetzt. Lange Dateinamen sind aber erlaubt.  */
/*                                                                   */
/* Input:                                                            */
/*   InputFile              Name der Eingabedatei                    */
/*   OutputFile             Name der Ausgabedatei                    */
/*   /Replace               Abfrage wegen existierender Ausgabedatei */
/*                          unterdrcken (fr Batch etc.)            */
/*   /Noreplace             Existierende Ausgabedatei niemals        */
/*                          berschreiben (fr Batch etc.)           */
/* Aenderungen:                                                      */
/*   Datum       Bemerkungen                                         */
/*   ----------  --------------------------------------------------- */
/*   02.12.1994  Korrekturen an Kommentierung und Usage-Info-Proc.   */
/*   17.10.1993  Korrekturen an der Kommentierung                    */
/*   03.10.1993  Nachbesserung EOF-Erkennung u. etw. Tuning          */
/*   02.10.1993  Einsatz von PARSE VAR statt SUBSTR() wg. Tuning     */
/*   01.10.1993  Beschleunigung des Verarbeitungsalgorithmus         */
/*   29.09.1993  Verbesserung der Benutzerfhrung. '/R' eingebaut    */
/*   28.09.1993  erste Version                                       */
/*********************************************************************/

CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

/* Beendigungscode initialisieren. Bei problemloser Verarbeitung     */
/* bleibt es bei 0.                                                  */
Severity = 0

/* Kommandozeile auswerten. Wenn zu wenig oder zu viel Parameter     */
/* angegeben wurden, endet die Verarbeitung mit einem Hilfetext.     */
/* Anmerkung: 'Rest' enthlt den gesamten Rest der Parameterzeile    */
/* inclusive fhrender Blanks nach dem ersten Blank nach dem letzten */
/* Token, das einer vorhergehenden Variable zugewiesen wurde. Darum  */
/* ist STRIP() erforderlich, damit ABBREV() funktioniert.            */
IF ARG() = 0 THEN SIGNAL ArgErrorExit
PARSE ARG InputFile OutputFile Rest
IF SUBSTR(OutputFile, 1, 1) == '/' THEN SIGNAL ArgErrorExit
IF ABBREV('/REPLACE', TRANSLATE(STRIP(Rest, 'B')), 2) THEN DO
  ReplaceIt = 1
  Rest = ''
  END  /* Do */
ELSE IF ABBREV('/NOREPLACE', TRANSLATE(STRIP(Rest, 'B')), 2) THEN DO
  ReplaceIt = -1
  Rest = ''
  END  /* Do */
ELSE ReplaceIt = 0
IF OutputFile = '' | Rest \= '' THEN SIGNAL ArgErrorExit

/* Existiert die Eingabedatei? Wenn nein: Ende mit Beendigungscode   */
/* 12 (ernster Fehler). Sonst: Verarbeitung fortsetzen.              */
Input = STREAM(InputFile, 'C', 'QUERY EXISTS')
IF Input == '' THEN DO
  SAY 'Datei ['InputFile'] nicht gefunden.'
  Severity = 12
  SIGNAL ErrorExit
  END

/* Existiert die Ausgabedatei? Falls ja, mu durch eine Abfrage      */
/* geklrt werden, ob die Datei berschrieben werden soll. Falls     */
/* nicht berschrieben werden soll, wird die Verarbeitung beendet.   */
/* Andernfalls wird die Datei gelscht und anschlieend mit der      */
/* Lnge 0 erneut angelegt. Die Abfrage kann durch den Parameter     */
/* '/Replace' an der dritten Position vermieden werden.              */
/* Sind Eingabe- und Ausgabedatei gleichnamig, bricht die            */
/* Verarbeitung ab, um Datenverlust zu vermeiden.                    */
Output = STREAM(OutputFile, 'C', 'QUERY EXISTS')
IF Output \= '' THEN DO
  IF Output == Input THEN DO
    SAY 'Eingabe- und Ausgabedateiname mssen verschieden sein.'
    Severity = 8
    SIGNAL ErrorExit
    END
  IF ReplaceIt = 1 THEN DO
    Key = 'J'
    SAY 'Die Ausgabedatei wird ersetzt.'
    END
  ELSE IF ReplaceIt = 0 THEN DO
    SAY "Die Datei ["Output"] existiert bereits."
    CALL CHAROUT , "Soll sie berschrieben werden? (Ja/Nein) "
    PARSE UPPER VALUE SysGetKey('NOECHO') WITH Key
    IF Key = 'J' THEN SAY 'Ja'; ELSE SAY 'Nein'
    END
  ELSE DO
    SAY 'Die Ausgabedatei existiert',
        'bereits und wird nicht berschrieben.'
    Severity = 4
    SIGNAL ErrorExit
    END
  IF Key = 'J' THEN DO
    CALL SysFileDelete Output
    IF Result \= 0 THEN DO
      SAY 'Problem beim Lschen der Datei ['Output']'
      SAY ErrorReport(Result)
      Severity = 12
      SIGNAL ErrorExit
      END
    END
  ELSE DO
    SAY "Programm durch Benutzer abgebrochen."
    Severity = 4
    SIGNAL ErrorExit
    END
  END
ELSE Output = TRANSLATE(OutputFile)

/* Versuch, die Ausgabedatei mit Lnge 0 anzulegen */
CALL CHAROUT Output
Output = STREAM(Output, 'C', 'QUERY EXISTS')
IF Output = '' THEN DO
  SAY "Die Datei ["OutputFile"] kann nicht angelegt werden."
  Severity = 12
  SIGNAL ErrorExit
  END

/* Bis hierher alles o.k. Die Verarbeitung beginnt. */
SAY "Eingabe: ["Input"]"
SAY "Ausgabe: ["Output"]"

/* Daten einlesen und ggf. EOF entfernen */
SAY "Lese Eingabe ..."
PARSE VALUE CHARIN(Input, 1, CHARS(Input)) WITH InputData '1A'x .

/* Hauptschleife: Vor jedem Durchlauf werden die (verbliebenen)     */
/* Eingabedaten in drei Teile geteilt: Anteil vor dem nchsten      */
/* LF, LF selbst und Anteil nach dem LF (= verbleibende Daten).     */
/* Solange noch Daten im Eingabepuffer stehen, werden LFs, die      */
/* nicht auf CRs folgen, in CRLFs umgewandelt: "Korrekt" mit CRLF   */
/* formatierte Daten bleiben unverndert.                           */
SAY 'Verarbeite Eingabedaten ...'
OutputData = ''
NumLines = 0
DO WHILE InputData \== ''
  PARSE VAR InputData,
            Head '0A'x -1 LastOfHead +1 FirstOfTail +1 InputData
  OutputData = OutputData || Head
  IF FirstOfTail == '0A'x THEN DO
    NumLines = NumLines + 1
    CALL CHAROUT , '1B'x || '[80D'NumLines 'Zeilen'
    IF LastOfHead == '0D'x THEN LastOfHead = ''
    OutputData = OutputData || LastOfHead'0A'x
    END
  ELSE OutputData = OutputData || FirstOfTail
  END
SAY " verarbeitet."

SAY "Schreibe Ausgabe ..."
CALL CHAROUT Output, OutputData, 1      /* Daten auf Platte schreiben */
CALL CHAROUT Output                     /* Ausgabedatei schlieen     */

/* Alles klar. Erfolgreiches Programmende. */
SAY 'Fertig.'
SIGNAL NormalExit

/*####################################################################*/

/* ErrorReport ordnet eine Beschreibung fr OS-Fehlercodes zu und     */
/* gibt diese als Resultat-Zeichenkette an den Aufrufer zurck.       */
ErrorReport: procedure
ErrNum = ARG(1)
ErrMsg.    = 'Unbekannter Fehler'
ErrMsg.0   = 'Lschvorgang erfolgreich'
ErrMsg.2   = 'Datei nicht gefunden'
ErrMsg.3   = 'Dateipfad nicht gefunden'
ErrMsg.5   = 'Zugriff verweigert'
ErrMsg.26  = 'Dateisystem ungeeignet'
ErrMsg.32  = 'Datei wird von einem anderen Proze benutzt'
ErrMsg.36  = 'Puffer fr gemeinsamen Zugriff nicht verfgbar'
ErrMsg.87  = 'Ungltiger oder fehlerhafter Parameter beim Aufruf'
ErrMsg.206 = 'Dateiname zu lang'
RETURN ErrMsg.ErrNum '(RC='ErrNum').'

/* Diese Routine wird angesprungen, wenn ein Fehler bei der Aus-      */
/* wertung der Aufrufparameter festgestellt wird. Sie gibt die ersten */
/* 23 Zeilen des Programmtextes aus, die als Kommentarkopf und Doku-  */
/* mentation fr den Benutzer angelegt sind.                          */
ArgErrorExit:
CALL SysCls
L1 = SOURCELINE(1)
DO n = 1 TO 23
  Ln = SOURCELINE(n)
  IF \ABBREV(Ln, '/*') THEN LEAVE
  SAY Ln
  END
CALL CHAROUT , L1
Severity = 1
SIGNAL NormalExit

/* Tritt ein Verarbeitungsfehler mit einem Severity-Code > 1 auf,     */
/* dann wird diese Routine angesprungen.                              */
ErrorExit:
SAY 'Programmabbruch (USER-RC='Severity').'

/* Diese Exit-Routine wird bei jedem Aufruf einmal durchlaufen.       */
/* Sie bergibt den Severity-Code an den Aufrufer von UX2OS.CMD,      */
/* der diesen als EXIT-Code auswerten kann.                           */
/* Severity  Bedeutung                                                */
/* --------  -------------------------------------------------------- */
/*      0    erfolgreiche Verarbeitung                                */
/*      1    fehlerhafte oder inkonsistente Aufrufparameter           */
/*      4    Programmabbruch wg. existierender Ausgabedatei /NOREPLAC */
/*      8    Abbruch wg. identischer Ein- und Ausgabedatei            */
/*     12    Dateifehler bei lschen, ffnen, lesen, schreiben etc.   */
NormalExit:
EXIT Severity

